;;; yf-NEWS-to-TODO.el ---                           -*- lexical-binding: t; -*-

;; Copyright (C) 2014  Nicolas Richard

;; Author: Nicolas Richard <theonewiththeevillook@yahoo.fr>
;; Keywords: 

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; 

;;; Code:

(defun yf/NEWS-to-TODO-current-headline-has-children ()
  ;; outline-level is also a function, but meant as a default of the
  ;; variable. Either way, it is assumed that the match data reflects
  ;; outline-regexp. This is ok if called with outline-map-region or
  ;; after outline-next-heading.
  (let ((level (funcall outline-level)))
    (save-excursion
      (save-match-data
        (outline-next-heading)
        (> (outline-level) level)))))

(defun yf-NEWS-to-TODO-current-heading-has-marker ()
  (member
   ;; assumed at begin of heading
   (buffer-substring (- (point) 4) (1- (point)))
   '("---" "+++")))

(defun yf-NEWS-to-TODO-convert-current-heading-todo ()
  (save-excursion
    (skip-chars-forward "*") ; assumed at begin of heading.
    (insert " TODO")))

(defun yf/NEWS-to-TODO-make-current-heading-todo-maybe ()
  (when (not
         (or
          (eq (char-after) ?)
          (yf/NEWS-to-TODO-current-headline-has-children)
          (yf-NEWS-to-TODO-current-heading-has-marker)))
    (yf-NEWS-to-TODO-convert-current-heading-todo)))

(defun yf/NEWS-to-TODO ()
  (interactive)
  ;; unmarked items are either TODO elements or from a previous
  ;; release. We now guess where those of the first kind are.
  (let ((beg
         (progn
           (goto-char (point-min))
           (re-search-forward "^* ")
           (point)))
        (end
         ;; it would be surprising if a whole heading had no markers
         ;; at all and yet did not pertain to a previous emacs release
         (progn
           (goto-char (point-max))
           (re-search-backward "^[+-]\\{3\\}$") ; last marked item
           (re-search-forward "^* ")            ; next main heading
           (point))))
    (outline-map-region #'yf/NEWS-to-TODO-make-current-heading-todo-maybe beg end)))

(provide 'yf-NEWS-to-TODO)
;;; yf-NEWS-to-TODO.el ends here
